@@ -1,6 +1,7 @@
 """
 Contains utilities to watch files and run commands on file updates
 """
+import datetime
 import logging
 
 from watchdog.observers import Observer
@@ -12,7 +13,7 @@ from util import get_files, register_keep_alive, run_function_list
 logger = logging.getLogger(__name__)
 
 
-def watch(fns, path, files=None, run_immediately=True, recursive=False, keep_alive=True, handler=None):
+def watch(fns, path, files=None, run_immediately=True, recursive=False, throttle=1, keep_alive=True, handler=None):
     """
 
     Runs the given function list when items at path change
@@ -26,6 +27,7 @@ def watch(fns, path, files=None, run_immediately=True, recursive=False, keep_ali
                   If this is a dict and no path is specified, path will be passed in to files.
     :param run_immediately: True to run all functions immediately
     :param recursive: true to watch files in path recursively
+    :param throttle: number of seconds to throttle each run
     :param keep_alive: true to keep build script alive for this watch
     :param handler: the watch handler to use
 
@@ -55,7 +57,7 @@ def watch(fns, path, files=None, run_immediately=True, recursive=False, keep_ali
                     f['path'] = path
 
     f = get_files(files) if files else None
-    handler = handler or WatchHandler(fns, predicate=lambda event: not f or event.src_path in f)
+    handler = handler or WatchHandler(fns, throttle=throttle, predicate=lambda event: not f or event.src_path in f)
 
     observer = Observer()
     observer.schedule(
@@ -74,7 +76,7 @@ class WatchHandler(FileSystemEventHandler):
     """
     Default watch handler class
     """
-    def __init__(self, fns, callback=None, predicate=None):
+    def __init__(self, fns, throttle=1, callback=None, predicate=None):
         """
         initialize watch handler
         :param fns: function list to run on event
@@ -83,6 +85,8 @@ class WatchHandler(FileSystemEventHandler):
         self.fns = fns
         self.callback = callback
         self.predicate = predicate
+        self.throttle = throttle
+        self.last_run = None
 
     def dispatch(self, event):
         """Dispatches events to the appropriate methods.
@@ -92,9 +96,10 @@ class WatchHandler(FileSystemEventHandler):
         :type event:
             :class:`FileSystemEvent`
         """
-        logger.debug("Got {} watch dispatch for path {}. Event: ".format(event.event_type,
+        logger.debug("Got {} watch dispatch for path {}. Event Type: {}".format(event.event_type,
                                                                          event.src_path,
-                                                                         str(event)))
+                                                                         str(event.event_type)))
+
         if self.predicate and not self.predicate(event):
             return
         super(WatchHandler, self).dispatch(event)
@@ -103,9 +108,16 @@ class WatchHandler(FileSystemEventHandler):
         """
         Run the functions on any event
         """
-        logger.debug("Got {} event for path {}. Event: ".format(event.event_type,
-                                                                event.src_path,
-                                                                str(event)))
+        logger.info("Got {} event for path {}. Event Type: {}".format(event.event_type,
+            event.src_path, str(event.event_type)))
+
+        if self.last_run:
+            if (datetime.datetime.now() - self.last_run).total_seconds() < self.throttle:
+                logger.info("Throttle time not elapsed - not running event")
+                return
+
+        if not self.last_run:
+            self.last_run = datetime.datetime.now()
 
         try:
             def predicate(fn):
